@@ -108,11 +108,7 @@ _jpeg_com_process(int fd, int len, struct lms_string_size *comment)
         comment->str[len] = '\0';
     comment->len = len;
 
-    lms_strstrip(comment->str, &comment->len);
-    if (comment->len == 0) {
-        free(comment->str);
-        comment->str = NULL;
-    }
+    lms_string_size_strip_and_free(comment);
 
     return 0;
 }
@@ -313,11 +309,7 @@ _exif_text_encoding_get(int fd, unsigned int count, int offset, struct lms_strin
     s->str[count] = '\0';
     s->len = count;
 
-    lms_strstrip(s->str, &s->len);
-    if (s->len == 0) {
-        free(s->str);
-        s->str = NULL;
-    }
+    lms_string_size_strip_and_free(s);
 
     return 0;
 }
@@ -348,11 +340,7 @@ _exif_text_ascii_get(int fd, unsigned int count, int offset, struct lms_string_s
     s->str[count - 1] = '\0';
     s->len = count - 1;
 
-    lms_strstrip(s->str, &s->len);
-    if (s->len == 0) {
-        free(s->str);
-        s->str = NULL;
-    }
+    lms_string_size_strip_and_free(s);
 
     return 0;
 }
@@ -551,6 +539,7 @@ static int
 _jfif_data_get(int fd, int len, struct lms_image_info *info)
 {
     unsigned char buf[4];
+    int new_len;
 
     memset(info, 0, sizeof(*info));
     info->orientation = 1;
@@ -566,16 +555,22 @@ _jfif_data_get(int fd, int len, struct lms_image_info *info)
         return -2;
     }
 
-    len = ((buf[2] << 8) | buf[3]);
+    new_len = ((buf[2] << 8) | buf[3]);
     if (buf[0] != 0xff) {
         fprintf(stderr, "ERROR: expected 0xff marker, got %#x\n", buf[0]);
         return -3;
     }
 
     if (buf[1] == JPEG_MARKER_EXIF)
-        return _exif_data_get(fd, len, info);
-    else
+        return _exif_data_get(fd, new_len, info);
+    else {
+        /* rollback to avoid losing initial frame */
+        if (lseek(fd, - len - 2, SEEK_CUR) == -1) {
+            perror("lseek");
+            return -1;
+        }
         return _jpeg_info_get(fd, len, info);
+    }
 }
 
 static const char _name[] = "jpeg";
